@@ -22,7 +22,24 @@
 #endif
 
 
+/* Some systems get flicker if we don't turn on OpenGL double-buffering.
+   I have reports of flicker on 10.7.5 Macbooks, but I can't reproduce
+   it on a 10.8.3 iMac-2010.
+ */
+#define DO_DOUBLE_BUFFER
+
+
+/* On a 10.6 iMac-2008, I was seeing the behavior that a transparent
+   Dali Clock window on top of iTunes playing videos would flicker.  At
+   the time I believed that OpenGL double-buffering didn't fix it, but
+   that alternating between two different texture IDs, and only writing
+   to the one not currently on screen, did fix it.
+*/
+#undef DO_DOUBLE_TEXTURE
+
+
 static void check_gl_error (const char *type);
+static void log_gl_error (const char *type, GLenum error);
 
 #ifdef USE_IPHONE
 static void rgb_to_hsv (CGFloat r, CGFloat g, CGFloat b, 
@@ -244,7 +261,9 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
       NSOpenGLPFAColorSize, 24,
       NSOpenGLPFAAlphaSize, 8,
       NSOpenGLPFADepthSize, 0, //16,  we don't need a depth buffer at all.
-//    NSOpenGLPFADoubleBuffer,  // does double buffering help at all?
+#  ifdef DO_DOUBLE_BUFFER
+      NSOpenGLPFADoubleBuffer,
+#  endif
       0 };
 
     NSOpenGLPixelFormat *pixfmt = 
@@ -401,7 +420,12 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
 
 # endif // USE_IPHONE
 
+# ifdef DO_DOUBLE_TEXTURE
   glGenTextures (2, textures);
+# else
+  glGenTextures (1, textures);
+# endif
+
   check_gl_error ("glGenTextures");
 
   return self;
@@ -909,15 +933,26 @@ FAIL:
 #endif /* 0 */
 
 
-/* The big kahuna refresh method.  Draw the current contents of the bitmap
+
+/**************************************************************************
+ **************************************************************************
+
+   The big kahuna refresh method.  Draw the current contents of the bitmap
    onto the window.  Sounds easy, doesn't it?
- */
+
+ **************************************************************************
+ **************************************************************************/
+
 - (void)drawRect:(NSRect)rect
 {
   NSRect framerect = [self frame];
   NSRect framerect2 = framerect;
   NSRect torect;
 
+  /**************************************************************************
+     Compute the desired size and rotation of the destination rectangle.
+   **************************************************************************/
+
   if (config.width <= 0 || config.height <= 0) abort();
 
   if (!fg || !bg) return; // Called too early somehow?
@@ -1000,45 +1035,149 @@ FAIL:
     torect.size.height = framerect2.size.width / img_aspect;
   }
 
-  // The animation slows down a lot if we use truly gigantic numbers,
-  // so limit the number size in screensaver-mode.
-  //
-  if (constrainSizes) {
-    int maxh = (config.time_mode == SS ? 512 : /*256*/ 200);
-    if (torect.size.height > maxh) {
-      torect.size.height = maxh;
-      torect.size.width  = maxh * img_aspect;
+
+  /**************************************************************************
+     Choose the size of the texture
+   **************************************************************************/
+
+  char err[100];
+  int retries = 0;
+
+  while (1) {	// Loop trying smaller sizes
+
+    strcpy (err, "glTexImage2D");
+
+    // The animation slows down a lot if we use truly gigantic numbers,
+    // so limit the number size in screensaver-mode.  Note that this
+    // limits the size of the output quad, not the size of the texture.
+    //
+    if (constrainSizes) {
+      int maxh = (config.time_mode == SS ? 512 : /*256*/ 200);
+      if (torect.size.height > maxh) {
+        torect.size.height = maxh;
+        torect.size.width  = maxh * img_aspect;
+      }
+    }
+
+    // put a 10% margin between the numbers and the edge of the window.
+    torect.size.width  *= 0.9;
+    torect.size.height *= 0.9;
+
+    // center it in the window
+    //
+    torect.origin.x = (framerect.size.width  - torect.size.width ) / 2;
+    torect.origin.y = (framerect.size.height - torect.size.height) / 2;
+
+    // Don't allow the top of the number to be off screen (iPhone 5, sec only)
+    //
+    if (torect.origin.x < 0) {
+      float r = ((torect.size.width + torect.origin.x) / torect.size.width);
+      torect.size.width  *= r;
+      torect.size.height *= r;
+      torect.origin.x = (framerect.size.width  - torect.size.width ) / 2;
+      torect.origin.y = (framerect.size.height - torect.size.height) / 2;
+    }
+
+
+  /**************************************************************************
+     Set the current GL context before talking to OpenGL
+   **************************************************************************/
+
+# ifdef USE_IPHONE
+  [EAGLContext setCurrentContext:ogl_ctx];
+  glBindFramebufferOES(GL_FRAMEBUFFER_OES, gl_framebuffer);
+# else /* !USE_IPHONE */
+  [ogl_ctx makeCurrentContext];
+  [ogl_ctx setView:self];
+  [ogl_ctx update];
+# endif /* !USE_IPHONE */
+
+
+  /**************************************************************************
+     Bind the bitmap to a texture
+   **************************************************************************/
+
+    // We alternate between two different texture IDs, only writing to the one
+    // that is not currently on screen.  If we don't do this, then the window
+    // flickers if it is above another window that has video playing in it.
+
+    sprintf (err + strlen(err), " #%d %dx%d", 
+             retries, config.width2, config.height2);
+
+    if (! config.bitmap) abort();
+    glBindTexture (GL_TEXTURE_2D, textures[0]);
+    glTexImage2D (GL_TEXTURE_2D, 0, GL_LUMINANCE_ALPHA,
+                  config.width2, config.height2, 0,
+                  GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, config.bitmap);
+
+    GLenum errn = glGetError();
+    if (errn == GL_NO_ERROR) {			// Texture succeeded.
+      break;
+    } else if (errn == GL_INVALID_VALUE) {	// Texture too large. Retry.
+
+      log_gl_error (err, errn);
+      retries++;
+
+      // Reduce the target size of the texture.
+      // If it's insanely small, abort.
+
+      unsigned int ow = config.width2;
+      unsigned int oh = config.height2;
+      int toosmall = 20;
+      int size_tries = 0;
+
+      while (config.height2 > toosmall &&
+             ow == config.width2 &&
+             oh == config.height2) {
+        config.height -= 4;
+        render_bitmap_size (&config, 
+                            &config.width, &config.height,
+                            &config.width2, &config.height2);
+        if (size_tries++ > 2000) abort();  // sanity
+      }
+
+      if (config.height2 <= toosmall) abort();
+
+      // Must re-render if size changed.
+      if (config.bitmap) free (config.bitmap);
+      config.bitmap = 0;
+      if (config.render_state) render_free (&config);
+      render_init (&config);
+
+    } else if (errn != GL_NO_ERROR) {		// Unknown error.
+      log_gl_error (err, errn);
+      abort();
     }
   }
 
-  // put a 10% margin between the numbers and the edge of the window.
-  torect.size.width  *= 0.9;
-  torect.size.height *= 0.9;
+  if (retries > 0)
+    NSLog (@"%s: Succeded: %s", progname, err);
 
-  // center it in the window
-  //
-  torect.origin.x = (framerect.size.width  - torect.size.width ) / 2;
-  torect.origin.y = (framerect.size.height - torect.size.height) / 2;
 
-  // Don't allow the top of the number to be off screen (iPhone 5, sec only)
-  //
-  if (torect.origin.x < 0) {
-    float r = ((torect.size.width + torect.origin.x) / torect.size.width);
-    torect.size.width  *= r;
-    torect.size.height *= r;
-    torect.origin.x = (framerect.size.width  - torect.size.width ) / 2;
-    torect.origin.y = (framerect.size.height - torect.size.height) / 2;
-  }
+  /**************************************************************************
+     Finish texture initialization
+   **************************************************************************/
 
-  // Set the viewport and projection matrix
+# ifdef DO_DOUBLE_TEXTURE
+  GLuint swap = textures[0];
+  textures[0] = textures[1];
+  textures[1] = swap;
+# endif
 
-#ifdef USE_IPHONE
-  [EAGLContext setCurrentContext:ogl_ctx];
-  glBindFramebufferOES(GL_FRAMEBUFFER_OES, gl_framebuffer);
-#else /* !USE_IPHONE */
-  [ogl_ctx makeCurrentContext];
-  [ogl_ctx setView:self];
-#endif /* !USE_IPHONE */
+  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
+  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
+  check_gl_error ("glTexParameteri");
+
+  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+  glEnable (GL_BLEND);
+  glEnable (GL_TEXTURE_2D);
+
+
+  /**************************************************************************
+     Set the viewport and projection matrix
+   **************************************************************************/
 
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
@@ -1050,7 +1189,9 @@ FAIL:
   glLoadIdentity();
 
 
-  // Set the OpenGL foreground and background colors
+  /**************************************************************************
+     Set the foreground and background colors
+   **************************************************************************/
 
   CGFloat fgr, fgg, fgb, fga;
   CGFloat bgr, bgg, bgb, bga;
@@ -1070,29 +1211,10 @@ FAIL:
   glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   check_gl_error ("clear");
 
-  // We alternate between two different texture IDs, only writing to the one
-  // that is not currently on screen.  If we don't do this, then the window
-  // flickers if it is above another window that has video playing in it.
 
-  if (! config.bitmap) abort();
-  glBindTexture (GL_TEXTURE_2D, textures[0]);
-  glTexImage2D (GL_TEXTURE_2D, 0, GL_LUMINANCE_ALPHA,
-                config.width2, config.height2, 0,
-                GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, config.bitmap);
-  check_gl_error ("glTexImage2D");
-  GLuint swap = textures[0];
-  textures[0] = textures[1];
-  textures[1] = swap;
-
-  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
-  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
-  check_gl_error ("glTexParameteri");
-
-  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-  glEnable (GL_BLEND);
-  glEnable (GL_TEXTURE_2D);
+  /**************************************************************************
+     Create the quad
+   **************************************************************************/
 
   GLfloat qx = torect.origin.x;
   GLfloat qy = torect.origin.y;
@@ -1113,7 +1235,7 @@ FAIL:
     glTranslatef (-qw * ((GLfloat) config.left_offset / config.width),
                   0, 0);
 
-#ifndef USE_IPHONE
+# ifndef USE_IPHONE
   glBegin (GL_QUADS);
   glTexCoord2f (0,  0);  glVertex3f (qx,    qy,    0);
   glTexCoord2f (tw, 0);  glVertex3f (qx+qw, qy,    0);
@@ -1121,7 +1243,7 @@ FAIL:
   glTexCoord2f (0,  th); glVertex3f (qx,    qy+qh, 0);
   glEnd();
   check_gl_error ("quad");
-#else /* USE_IPHONE */
+# else /* USE_IPHONE */
   GLfloat vertices[] = {
     qx,    qy,    0,
     qx+qw, qy,    0,
@@ -1143,18 +1265,22 @@ FAIL:
   glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
   check_gl_error ("draw arrays");
 
-#endif /* USE_IPHONE */
+# endif /* USE_IPHONE */
+
+  /**************************************************************************
+     Flush bits to the screen.
+   **************************************************************************/
 
   glFinish();
 
-#ifndef USE_IPHONE
+# ifndef USE_IPHONE
   [ogl_ctx flushBuffer];
   check_gl_error ("finish");
-#else /* USE_IPHONE */
+# else /* USE_IPHONE */
   glBindRenderbufferOES (GL_RENDERBUFFER_OES, gl_renderbuffer);
   [ogl_ctx presentRenderbuffer:GL_RENDERBUFFER_OES];
   check_gl_error ("finish");
-#endif /* USE_IPHONE */
+# endif /* USE_IPHONE */
 }
 
 
@@ -1356,45 +1482,51 @@ FAIL:
 
 - (void)showBlurb:(NSString *)text
 {
-  CGFloat margin = [self frame].size.width * 0.05;
-  CGRect frame;
+  CGRect frame = [self frame];
   CGFloat rot;
-  CGFloat pt = 14;
-  UIFont *font = [UIFont boldSystemFontOfSize:pt];
-  CGFloat width;
-  CGFloat height = [text sizeWithFont:font
-                         constrainedToSize:CGSizeMake(10240,10240)].height;
+  int size = ([self frame].size.width >= 768 ? 24 : 14);
+  UIFont *font = [UIFont boldSystemFontOfSize: size];
+  CGSize tsize = [text sizeWithFont:font
+                  constrainedToSize:CGSizeMake(frame.size.width,
+                                               frame.size.height)];
 
-  height += 20; // Don't know how to find the inner margins of the UITextView.
+  // Don't know how to find inner margin of UITextView.
+  CGFloat margin = 10;
+  tsize.width  += margin * 4;
+  tsize.height += margin * 2;
 
-  // Handle rotation here too.  This is insane.
-  //
-  switch ([[UIDevice currentDevice] orientation]) {
-  case UIDeviceOrientationLandscapeRight:
+  frame = CGRectMake (0, 0, tsize.width, tsize.height);
+
+  UIInterfaceOrientation orient =
+    // Why are both of these wrong when starting up rotated??
+    [[UIDevice currentDevice] orientation];
+    // [rootViewController interfaceOrientation];
+
+  /* Get the text oriented properly, and move it to the bottom of the
+     screen.
+   */
+  switch (orient) {
+  case UIDeviceOrientationLandscapeRight:     
     rot = -M_PI/2;
-    width = [self frame].size.height - margin*2;
-    frame = CGRectMake (([self frame].size.width / 2) - height - margin*2,
-                        ([self frame].size.height - height) / 2,
-                        width, height);
+    frame.origin.x = ([self frame].size.width
+                      - (tsize.width - tsize.height) / 2
+                      - tsize.height);
+    frame.origin.y = ([self frame].size.height - tsize.height) / 2;
     break;
   case UIDeviceOrientationLandscapeLeft:
     rot = M_PI/2;
-    width = [self frame].size.height - margin*2;
-    frame = CGRectMake (-([self frame].size.width / 2) - margin,
-                        ([self frame].size.height - height) / 2,
-                        width, height);
+    frame.origin.x = -(tsize.width - tsize.height) / 2;
+    frame.origin.y = ([self frame].size.height - tsize.height) / 2;
     break;
-  case UIDeviceOrientationPortraitUpsideDown:
+  case UIDeviceOrientationPortraitUpsideDown: 
     rot = M_PI;
-    width = [self frame].size.width - margin*2;
-    frame = CGRectMake (margin, margin, width, height);
+    frame.origin.x = ([self frame].size.width  - tsize.width) / 2;
+    frame.origin.y = 0;
     break;
-  default: 
+  default:
     rot = 0;
-    width = [self frame].size.width - margin*2;
-    frame = CGRectMake (margin,
-                        [self frame].size.height - height - margin,
-                        width, height);
+    frame.origin.x = ([self frame].size.width  - tsize.width) / 2;
+    frame.origin.y =  [self frame].size.height - tsize.height;
     break;
   }
 
@@ -1413,6 +1545,13 @@ FAIL:
   aboutBox.text = text;
   aboutBox.editable = NO;
 
+  // Too subtle?
+  CALayer *textLayer = (CALayer *)[aboutBox.layer.sublayers objectAtIndex:0];
+  textLayer.shadowColor   = [UIColor blackColor].CGColor;
+  textLayer.shadowOffset  = CGSizeMake(0, 0);
+  textLayer.shadowOpacity = 1;
+  textLayer.shadowRadius  = 2;
+
   CABasicAnimation *anim = 
     [CABasicAnimation animationWithKeyPath:@"opacity"];
   anim.duration     = 0.3;
@@ -1428,7 +1567,7 @@ FAIL:
   [aboutBox addGestureRecognizer: tap];
   [tap release];
 
-  [self addSubview:aboutBox];
+  [[self superview] addSubview:aboutBox];
 }
 
 - (void) handleAboutTap
@@ -1659,12 +1798,13 @@ rgb_to_hsv (CGFloat R, CGFloat G, CGFloat B,
 #endif /* USE_IPHONE */
 
 
-/* report a GL error. */
+
+/* Prints the GL error to the system log.
+ */
 static void
-check_gl_error (const char *type)
+log_gl_error (const char *type, GLenum err)
 {
   char buf[100];
-  GLenum i;
   const char *e;
 
 # ifndef  GL_TABLE_TOO_LARGE_EXT
@@ -1677,8 +1817,8 @@ check_gl_error (const char *type)
 #  define GL_INVALID_FRAMEBUFFER_OPERATION 0x0506
 # endif
 
-  switch ((i = glGetError())) {
-    case GL_NO_ERROR: return;
+  switch (err) {
+    case GL_NO_ERROR:              return;
     case GL_INVALID_ENUM:          e = "invalid enum";      break;
     case GL_INVALID_VALUE:         e = "invalid value";     break;
     case GL_INVALID_OPERATION:     e = "invalid operation"; break;
@@ -1689,9 +1829,19 @@ check_gl_error (const char *type)
     case GL_TEXTURE_TOO_LARGE_EXT: e = "texture too large"; break;
     case GL_INVALID_FRAMEBUFFER_OPERATION: e = "invalid framebuffer op"; break;
     default:
-      e = buf; sprintf (buf, "unknown GL error 0x%04x", (int) i); break;
+      e = buf; sprintf (buf, "unknown GL error 0x%04x", (int) err); break;
   }
-  NSLog (@"%s GL error: %s", type, e);
-  abort();
+
+  NSLog (@"%s: GL error in %s: %s", progname, type, e);
 }
 
+
+/* Log a GL error and abort. */
+static void
+check_gl_error (const char *type)
+{
+  GLenum err = glGetError();
+  if (err == GL_NO_ERROR) return;
+  log_gl_error (type, err);
+  abort();
+}
