@@ -1,5 +1,5 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1999, 2001
+ * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1999, 2001, 2006
  *  Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -189,14 +189,21 @@ cycle_colors (Screen *screen, Colormap cmap,
               Window window, GC fg_gc, GC bg_gc)
 {
   Display *dpy = DisplayOfScreen(screen);
-  static int hue_tick;
-  hsv_to_rgb (hue_tick,
-              1.0, 1.0,
+  static int scale = 3;
+  static int hue_tick1 = -1, hue_tick2;
+
+  if (hue_tick1 < 0)
+    {
+      hue_tick1 = time ((time_t *) 0) % (360*scale);
+      hue_tick2 = (hue_tick1 + 180) % (360*scale);
+    }
+
+  hsv_to_rgb (hue_tick1/scale, 1.0, 1.0,
               &fg_color->red, &fg_color->green, &fg_color->blue);
-  hsv_to_rgb ((hue_tick + 180) % 360,
-              1.0, 1.0,
+  hsv_to_rgb (hue_tick2/scale, 1.0, 0.4,
               &bg_color->red, &bg_color->green, &bg_color->blue);
-  hue_tick = (hue_tick+1) % 360;
+  hue_tick1 = (hue_tick1+2) % (360*scale);
+  hue_tick2 = (hue_tick2+3) % (360*scale);
 
   if (! no_writable_cells)
     {
