@@ -1,6 +1,5 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
- *               2002, 2003, 2006 Jamie Zawinski <jwz@jwz.org>
+ * Copyright (c) 1991-2010 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -82,6 +81,8 @@ static XrmOptionDescRec options [] = {
   /* options from digital.c */
   { "-12",              ".mode",                XrmoptionNoArg, "12" },
   { "-24",              ".mode",                XrmoptionNoArg, "24" },
+  { "-showdate",        ".showdate",            XrmoptionNoArg, "True" },
+  { "-showtime",        ".showdate",            XrmoptionNoArg, "False" },
   { "-datemode",        ".datemode",            XrmoptionSepArg, 0 },
   { "-mm/dd/yy",        ".datemode",            XrmoptionNoArg, "MM/DD/YY" },
   { "-dd/mm/yy",        ".datemode",            XrmoptionNoArg, "DD/MM/YY" },
@@ -97,7 +98,6 @@ static XrmOptionDescRec options [] = {
   { "-builtin2",        ".font",                XrmoptionNoArg, "BUILTIN2" },
   { "-builtin3",        ".font",                XrmoptionNoArg, "BUILTIN3" },
   { "-builtin4",        ".font",                XrmoptionNoArg, "BUILTIN4" },
-  { "-builtin5",        ".font",                XrmoptionNoArg, "BUILTIN5" },
   { "-memory",          ".memory",              XrmoptionSepArg, 0 },
   { "-sleaze-level",    ".memory",              XrmoptionSepArg, 0 },
   { "-oink",            ".memory",              XrmoptionNoArg, "Medium" },
@@ -444,10 +444,12 @@ where options include\n\
 \n\
   -12                           Display twelve hour time (default).\n\
   -24                           Display twenty-four hour time.\n\
+  -showtime                     Show the time initially (default).\n\
+  -showdate                     Show the date initially.\n\
   -seconds                      Display seconds (default).\n\
   -noseconds                    Don't display seconds.\n\
-  -cycle                        Do color-cycling.\n\
-  -nocycle                      Don't do color-cycling (default).\n\
+  -cycle                        Do color-cycling (default).\n\
+  -nocycle                      Don't do color-cycling.\n\
   -countdown <date>             Display a countdown instead of a clock.\n\
                                 Run `-countdown foo' to see date syntax.\n\
   -display <host:dpy>           The display to run on.\n\
@@ -470,7 +472,7 @@ where options include\n\
   fprintf (stderr, "\
   -font or -fn <font>           Name of an X font or built-in font to use.\n\
                                 There are six builtin fonts, named\n\
-                                \"BUILTIN0\", through \"BUILTIN5\", with #2\n\
+                                \"BUILTIN0\", through \"BUILTIN4\", with #2\n\
                                 being the default.\n\
   -builtin0                     Same as -font BUILTIN0.\n\
   -builtin1                     Same as -font BUILTIN1.\n\
@@ -478,7 +480,6 @@ where options include\n\
   -builtin                      Same as -font BUILTIN.\n\
   -builtin3                     Same as -font BUILTIN3.\n\
   -builtin4                     Same as -font BUILTIN4.\n\
-  -builtin5                     Same as -font BUILTIN5.\n\
 ");
 #else /* !BUILTIN_FONTS */
   fprintf (stderr, "\
