@@ -223,11 +223,11 @@ copy_frame (struct state *state, FrameHandle from_h, FrameHandle to_h)
 
 /* Returns the largest font that will fit on the screen.
  */
-static struct raw_number *
+static const struct raw_number *
 pick_font (struct state *state)
 {
   int nn, cc;
-  struct raw_number *num;
+  const struct raw_number *num;
 
   if (state->seconds_only_p)
     nn = 2, cc = 0;
@@ -236,8 +236,8 @@ pick_font (struct state *state)
 
 # define CHECK(WHICH) \
     num = get_raw_number_##WHICH (); \
-    if (state->win_width  * 1.1 >= ((num->width * nn) + (num->width * cc)) && \
-        state->win_height * 1.1 >= num->height) \
+    if (state->win_width  >= ((num[0].width * nn) + (num[10].width * cc)) && \
+        state->win_height >= num[0].height) \
       return num
   CHECK(3);
   CHECK(2);
@@ -253,7 +253,7 @@ static Boolean
 init_numbers (struct state *state)
 {
   int i;
-  struct raw_number *raw = pick_font (state);
+  const struct raw_number *raw = pick_font (state);
   if (!raw) return false;
 
   state->char_width  = raw[0].width;
@@ -1132,6 +1132,9 @@ app_handle_event (struct state *state, EventPtr event)
         if (! state->active_window_p)
           return true;
 
+        if (state->rom_version >= ROM_35)
+          WinScreenLock (winLockCopy);
+
         tick_sequence (state);
 
         if (redraw_p)
@@ -1145,6 +1148,9 @@ app_handle_event (struct state *state, EventPtr event)
 
         draw_clock (state);
 
+        if (state->rom_version >= ROM_35)
+          WinScreenUnlock ();
+
         if (redraw_p)
           sync_to_seconds ();
 
