@@ -1,5 +1,5 @@
 /* Dali Clock - a melting digital clock for Palm WebOS.
- * Copyright (c) 1991-2009 Jamie Zawinski <jwz@jwz.org>
+ * Copyright (c) 1991-2010 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -155,6 +155,7 @@ DaliClock.prototype.clock_reset = function() {
   }
 
   this.orig_frames    = new Array(8);  // what was there
+  this.orig_digits    = new Array(8);  // what was there
   this.current_frames = new Array(8);  // current intermediate animation
   this.target_frames  = new Array(8);  // where we are going
   this.target_digits  = new Array(8);  // where we are going
@@ -163,6 +164,7 @@ DaliClock.prototype.clock_reset = function() {
     var colonic_p = (i == 2 || i == 5);
     var empty = (colonic_p ? this.font.empty_colon : this.font.empty_frame);
     this.orig_frames[i]    = empty;
+    this.orig_digits[i]    = undefined;
     this.target_frames[i]  = empty;
     this.current_frames[i] = this.copy_frame (empty);
   }
@@ -461,11 +463,12 @@ DaliClock.prototype.start_sequence = function(date) {
   if (this.new_settings)
     this.settings_changed();
 
-  // Copy the (old) current_frames into the (new) orig_frames,
+  // Move the (old) current_frames into the (new) orig_frames,
   // since that's what's on the screen now.
   //
   for (var i = 0; i < this.current_frames.length; i++) {
     this.orig_frames[i] = this.current_frames[i];
+    this.orig_digits[i] = this.target_digits[i];
   }
 
   // generate new target_digits
@@ -548,6 +551,41 @@ DaliClock.prototype.tick_sequence = function() {
                    this.target_frames[i],
                    msecs);
   }
+
+  this.current_msecs = msecs;
+}
+
+
+// left_offset is so that the clock can be centered in the window
+// when the leftmost digit is hidden (in 12-hour mode when the hour
+// is 1-9).  When the hour rolls over from 9 to 10, or from 12 to 1,
+// we animate the transition to keep the digits centered.
+//
+DaliClock.prototype.compute_left_offset = function() {
+  var left_offset;
+  if (this.target_digits[0] == undefined &&	 // Fading in to no digit
+      this.orig_digits[1] == undefined)
+    left_offset = this.font.char_width / 2;
+  else if (this.target_digits[0] != undefined && // Fading in to a digit
+           this.orig_digits[1] == undefined)
+    left_offset = 0;
+  else if (this.orig_digits[0] != undefined &&	 // Fading out from digit
+           this.target_digits[1] == undefined)
+    left_offset = 0;
+  else if (this.orig_digits[0] != undefined &&	 // Fading out from no digit
+           this.target_digits[1] == undefined)
+    left_offset = this.font.char_width / 2;
+  else if (this.orig_digits[0] == undefined &&	 // Anim no digit to digit.
+           this.target_digits[0] != undefined)
+    left_offset = (this.font.char_width * (1000 - this.current_msecs) / 2000);
+  else if (this.orig_digits[0] != undefined &&	 // Anim digit to no digit.
+           this.target_digits[0] == undefined)
+    left_offset = this.font.char_width * this.current_msecs / 2000;
+  else if (this.target_digits[0] == undefined)	 // No anim, no digit.
+    left_offset = this.font.char_width / 2;
+  else						 // No anim, digit.
+    left_offset = 0;
+  return left_offset;
 }
 
 
@@ -555,7 +593,7 @@ DaliClock.prototype.tick_sequence = function() {
 //
 DaliClock.prototype.draw_clock = function() {
 
-  var x = 0;
+  var x = -this.compute_left_offset();
   var y = 0;
   this.ctx.clearRect (0, 0, this.canvas.width, this.canvas.height);
 
